(princ "\r|")

;PED CADD                                                            Version 2.0
;
;By:  Technical Software, Inc.                               PH:  1-900-820-ACAD
;     Christopher Heltzel
;     4555 Emery Industrial Pkwy., Suite 102
;     Cleveland, Ohio 44128
;
;File Name:  REV.LSP
;
;Contents:  C:REV, Rev_error_flag, Get_rev_attributes, Set_coordinates,
;           Create_revblk & Move_revisions.
;
;References:  $error$, Error1-2, Getatt, Change_att, Query_int, Query_string &
;             Get_date (All are located in VENDOR.LSP)
;
;Usage:  Type REV
;        Enter the detail number the revision references or press <RETURN> for
;           no detail reference.  1 to 999 or <RETURN> are valid entries.
;        Enter the Engineers Name and Drafters Initials. 0 to 13 characters are
;           allowed.
;        Enter the revision description. 1 to 34 character are required.
;
;Results:     Queries for Detail Number, Engineer/Drafter and Revision
;          Description.  If old REV blocks are present, Error5 results and the
;          routine is cancelled.  If REVISION is not a defined block it is
;          created.  Any REVISION blocks present are moved up one line and those
;          outside of the titleblock area are changed to the REV layer which is
;          typically frozen.  The latest revision number and current date are
;          deterimined and the sheet number is appended to the Detail Number if
;          one was entered.  This and the queried information are inserted and
;          the titleblbock revision level is updated.

(princ "\r/")

(defun C:REV (/ #error# clayer cmdecho blipmode highlight flag ss1)
  (setq #error# *error*
        *error* $error$
        clayer (getvar "clayer")
        cmdecho (getvar "cmdecho")
        blipmode (getvar "blipmode")
        highlight (getvar "highlight")
        flag nil
  )
  (setvar "cmdecho" 0)
  (setvar "blipmode" 0)
  (command "undo" "e" "undo" "m")                                               ;set markers for *error*
  (if (setq ss1 (ssget "x" (list (cons 2 "pedflag"))))
    (Rev1 ss1)
    (Rev2)
  )
  (command "layer" "s" clayer "")
  (setvar "blipmode" blipmode)
  (setvar "highlight" highlight)
  (setvar "cmdecho" cmdecho)
  (setq *error* #error#)
  (princ)
)

(princ "\r-")

(defun Rev1 (ss1 / ss form_ename detail date engineer description revision
  display_rev move_coord rev4_inp inp rot revision
)
  (if (setq ss (ssget "x" (list (cons 2 "TITLEBLK"))))
    (progn
      (setq form_ename (ssname ss 0))
      (if (setq flag (cdr (assoc 1 (entget (entnext (ssname ss1 0))))))               ;get code if pedflag exists
        (progn                                                                      ;pedflag present
          (princ "\n \nBeginning revision update...\n")
          (Get_rev_attributes)                                                      ;defines detail, date, engineer, & description
          (Set_rev1_coords)                                                         ;defines display_rev, move_coord, rev4_inp,
          ;                                                                          inp & rot
          (if (tblsearch "layer" "t")                                                   ;create layer if nonexistent
            (command "layer" "s" "t" "")
            (command "layer" "m" "t" "c" "9" "t" "l" "continuous" "t" "t" "t" "")
          )
          (if (not (tblsearch "block" "rev"))                                       ;create rev block if not present
            (cond
              ( (= flag "1")
                (Create_revblk '(27.5625 4.625) '(27.9375 4.625) '(28.4688 4.625)
                               '(28.8688 4.625) '(29.8063 4.625))
              )
              ( (= flag "2")
                (Create_revblk '(33.5 4.625) '(33.875 4.625) '(34.4063 4.625)
                               '(34.8063 4.625) '(35.7438 4.625))
              )
              ( (= flag "3")
                (Create_revblk '(4.875 8.5) '(5.25 8.5) '(5.7813 8.5)
                               '(6.1813 8.5) '(7.1188 8.5))
              )
              ( (= flag "4")
                (Create_revblk '(0.9375 1.625) '(1.3125 1.625) '(1.8438 1.625)
                               '(2.2438 1.625) '(3.1813 1.625))
          ) ) )
          (if (setq ss1 (ssget "x" (list (cons 2 "rev"))))
            (Move_revisions ss1)                                                                   ;moves earlier revs up unless none are present
            (setq revision "1")
          )
          (command "insert" "rev" inp "1" "1.00000001" rot revision detail date     ;Y factor offset to prevent explosion
            engineer description
          )
          (cond
            ((= flag "1") (Change_att 13 form_ename revision))
            ((= flag "2") (Change_att 15 form_ename revision))
            ((= flag "3") (Change_att 12 form_ename revision))
            ((= flag "4") (Change_att 12 form_ename revision))
          )
          (princ "\nRevision update complete.")
        )
        (princ "\nERROR: Drawing does not contain the PEDFLAG block.\n        REV.LSP cannot continue.")                 ;no pedflag
    ) )
    (princ "\nError: Drawing does not contain the TITLEBLK block.\n        REV.LSP cannot continue.")
) )

(princ "\r\\")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun Rev2 (/ form_name form_ename detail date engineer description revision
  display_rev move_coord rev4_inp inp rot ss1 revision
)
  (if (and (not (Error1-2)) (not (Rev_error_flag)))                                                    ;defines form_name & form_ename
    (progn
      (princ "\n \nBeginning revision update...\n")
      (Get_rev_attributes)                                                      ;defines detail, date, engineer, & description
      (Set_rev2_coords)                                                         ;defines display_rev, move_coord, rev4_inp,
      ;                                                                          inp & rot
      (if (tblsearch "layer" "t")                                                   ;create layer if nonexistent
        (command "layer" "s" "t" "")
        (command "layer" "m" "t" "c" "9" "t" "l" "continuous" "t" "t" "t" "")
      )
      (if (not (tblsearch "block" "revision"))                                  ;create rev block if not present
        (cond
          ( (= form_name "PEDFORM")
            (Create_revblk '(27.5625 4.625) '(27.9375 4.625) '(28.4688 4.625)
                           '(28.8688 4.625) '(29.8063 4.625))
          )
          ( (= form_name "30X42")
            (Create_revblk '(33.5625 4.625) '(33.9375 4.625) '(34.4688 4.625)
                           '(34.8688 4.625) '(35.8063 4.625))
          )
          ( (= form_name "8X11")
            (Create_revblk '(0.9375 1.625) '(1.3125 1.625) '(1.8438 1.625)
                           '(2.2438 1.625) '(3.1813 1.625))
          )
          ( (or (= form_name "M-PEDBOM") (= form_name "E-PEDBOM") (= form_name "PEDBOM"))
            (Create_revblk '(4.875 8.4375) '(5.25 8.4375) '(5.7813 8.4375)
                           '(6.1813 8.4375) '(7.1188 8.4375))
      ) ) )
      (if (setq ss1 (ssget "x" (list (cons 2 "revision"))))
        (Move_revisions ss1)                                                    ;moves earlier revs up if present, define revision
        (setq revision "1")                                                     ;if not present then rev number is 1
      )
      (command "insert" "revision" inp "1" "1.00000001" rot revision detail date;Y factor offset to prevent explosion
        engineer description)
      (Change_att 13 form_ename revision)                                       ;updates rev level in titleblock
) ) )

(princ "\r|")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Rev_error_flag) function
;
;Results:     Searches the drawing entity list for a REV block definition.  This
;          is an old block and therefore this routine may not produce the
;          results.  If the block is found an error message is displayed and T
;          is returned.  If a REV block is not found then nil is returned.

(defun Rev_error_flag (/ error_flag)
  (setq error_flag T)
  (if (ssget "x" (list (cons 2 "rev")))
    (princ "\nError5: Old revision blocks present.\n        This version of REV.LSP cannot be used.") ;(princ "\nError5: Old revison blocks present.  See PED Manual page ???.")
    (setq error_flag nil)
  )
  (setq error_flag error_flag)
)

(princ "\r/")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Get_rev_attributes) function
;
;Results:     Sets the variables Detail, Date, Engineer & Description. Returns
;          nil.

(defun Get_rev_attributes ()
  (setq detail (Query_int_nil 1 999 "\nDetail Number or <no reference>:  "))
  (if detail
    (setq detail (itoa detail))
    (setq detail "")
  )
  (setq date (Get_date))
  (setq engineer
    (Query_string 0 34 "\nEngineer's Name/Drafter's Initials:  ")
  )
  (setq description
    (Query_string 1 66 "\nRevision Description:  ")
) )

(princ "\r-")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Set_rev2_coords) function
;
;Results:     Sets variables Display_rev, Move_coord, Rev4_inp, Inp & Rot based
;          on the titleblock form used.

(defun Set_rev2_coords ()
  (cond
    ( (= form_name "PEDFORM")
      (setq display_rev 3                                                       ;number of revs to display -1
            move_coord '(0.0 0.25)                                              ;relative coord to move old revs
            rev4_inp 5.4375                                                     ;relevant insert coord (x or y) of 4th rev up
            inp '(27.375 4.5625)                                                ;insert point for rev 1
            rot "0"                                                             ;revison blk insertion rotation
    ) )
    ( (= form_name "30X42")
      (setq display_rev 3
            move_coord '(0.0 0.25)
            rev4_inp 5.4375
            inp '(33.375 4.5625)
            rot "0"
    ) )
    ( (= form_name "8X11")
      (setq display_rev 0
            move_coord '(0.0 0.25)
            rev4_inp 1.6875
            inp '(0.75 1.5625)
            rot "0"
    ) )
    ( (or (= form_name "M-PEDBOM") (= form_name "E-PEDBOM") (= form_name "PEDBOM"))
      (setq display_rev 3
            move_coord '(0.25 0.0)
            rev4_inp 5.5625
            inp '(4.6875 8.375)
            rot "270"
) ) ) )

(princ "\r\\")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Set_rev1_coords) function
;
;Results:     Sets variables Display_rev, Move_coord, Rev4_inp, Inp & Rot based
;          on the titleblock form used.
(defun Set_rev1_coords ()
  (cond
    ( (= flag "1")
      (setq display_rev 3                                                       ;number of revs to display -1
            move_coord '(0.0 0.25)                                              ;relative coord to move old revs
            rev4_inp 5.6875                                                     ;relevant insert coord (x or y) of 4th rev up
            inp '(27.375 4.8125)                                                ;insert point for rev 1
            rot "0"                                                             ;revison blk insertion rotation
    ) )
    ( (= flag "2")
      (setq display_rev 3
            move_coord '(0.0 0.25)
            rev4_inp 5.6875
            inp '(33.3125 4.8125)
            rot "0"
    ) )
    ( (= flag "3")
      (setq display_rev 3
            move_coord '(0.25 0.0)
            rev4_inp 5.8125
            inp '(4.9375 8.4375)
            rot "270"
    ) )
    ( (= flag "4")
      (setq display_rev 0
            move_coord '(0.0 0.25)
            rev4_inp 2.1875
            inp '(0.75 1.8125)
            rot "0"
) ) ) )

(princ "\r|")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Create_revblk <pt1> <pt2> <pt3> <pt4> <pt5>) function
;
;Results:     Sets current layer to T if it is defined, otherwise it defines the
;          T layer first.  Sets the style to XC.  Creates the attribute
;          definitions of Rev, Detail, Date, Initials and Description.  The five
;          parameters <pt1>...<pt5> are insertion points for the attributes.
;          A block called REVISION, composed of the five attributes is created.
;          The layer and style are set back to the original values and nil is
;          returned.

(defun Create_revblk (atin1 atin2 atin3 atin4 atin5 / sty ss1)
  (setq sty (getvar "textstyle"))
  (command "style" "c" "" "" "" "" "" "" "")
  (setvar "aflags" 0)
  (setq ss1 (ssadd))                                                             ;adds new att to selection set
  (command "attdef" "" "REV" "Revision Level" "" "c" atin1 "0.1" "0")
  (ssadd (entlast) ss1)
  (command "attdef" "" "DETAIL" "Detail Number" "" "c" atin2 "0.1" "0")
  (ssadd (entlast) ss1)
  (command "attdef" "" "DATE" "Date" "" "c" atin3 ".1" "0")
  (ssadd (entlast) ss1)
  (command "attdef" "" "INITIALS" "Engineer/Drafter" "" atin4 "0.1" "0")
  (ssadd (entlast) ss1)
  (command "attdef" "" "DESCRIPTION" "Detail Description" "" atin5 "0.1" "0")
  (ssadd (entlast) ss1)
  (princ "\n \nProcessing...\n")
  (if flag
    (command "block" "rev" inp ss1 "")
    (command "block" "revision" inp ss1 "")
  )
  (command "style" sty "" "" "" "" "" "" "")
)

(princ "\r/")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Move_revision <ss>) function
;
;Result:     The selection set <ss> composed of all REVISION blocks is moved up
;         one line.  The layer F is created if neccessary and then set as
;         current.  If there are more REVISION blocks than spaces on the boarder
;         then the blocks with insertion points outside of the boarder are
;         changed to the REV layer. nil is returned.

(defun Move_revisions (ss1 / len ss2 rev_inp)
  (command "move" ss1 "" move_coord "")
  (setq len (sslength ss1))
  (setq revision (itoa (1+ len)))                                               ;increments rev level
  (if (tblsearch "layer" "rev")                                                 ;create layer if nonexistant
    (command "layer" "f" "rev" "")
    (command "layer" "n" "rev" "c" "9" "rev" "l" "continuous" "rev" "f" "rev" "")
  )
  (if (> len display_rev)                                                       ;if there are more revs than
    (progn                                                                      ; can be displayed, then put
      (setq ss2 (ssadd))                                                        ; them on rev layer
      (while (> len 0)
        (setq len (1- len))
        (if (or (= form_name "M-PEDBOM") (= form_name "E-PEDBOM")
                (= form_name "PEDBOM") (= flag "3"))
          (setq rev_inp (cadr (assoc 10 (entget (ssname ss1 len)))))            ;X coord if PEDBOM
          (setq rev_inp (caddr (assoc 10 (entget (ssname ss1 len)))))           ;Y coord if any other
        )
        (if (> rev_inp rev4_inp)
          (ssadd (ssname ss1 len) ss2)                                          ;add rev if outside of titleblock
      ) )
      (command "change" ss2 "" "l" "rev")
) ) )

(princ "\r-")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Error1-2) function
;
;References:  Gettag & Enatt (located within this file)
;
;Results:     Determines the type of boarder and creates the global variables
;          Form_name and Form_ename.  Checks if the boarder is valid based on
;          the DRAWING-NO, SHEET and REVISION attribute tags.  If Form_name can
;          not be determined Error 1 results.  If it is not a valid ped boarder
;          Error 2 results.
;             Sets the global variables form_name and form_ename.  Returns nil
;          if either error results, otherwise T is retuned.

(defun Error1-2 (/ error_flag ss1)
  (setq error_flag T)
  (cond                                                                         ;conditional test to determine boarder type
    ( (setq ss1 (ssget "x" (list (cons 2 "PEDFORM"))))                          ; creates
     (setq form_name "PEDFORM")
    )
    ((setq ss1 (ssget "x" (list (cons 2 "30X42")))) (setq form_name "30X42"))
    ((setq ss1 (ssget "x" (list (cons 2 "8X11")))) (setq form_name "8X11"))
    ( (setq ss1 (ssget "x" (list (cons 2 "M-PEDBOM"))))
      (setq form_name "M-PEDBOM")
    )
    ( (setq ss1 (ssget "x" (list (cons 2 "E-PEDBOM"))))
      (setq form_name "E-PEDBOM")
    )
    ( (setq ss1 (ssget "x" (list (cons 2 "PEDBOM"))))
      (setq form_name "PEDBOM")
    )
    (T (princ "\nError1: Proper drawing form (Ex: PEDFORM) is not present.\n        REV.LSP cannot continue."))
;    (T (princ "\nError1: Proper drawing form (Ex: PEDFORM) is not present.  See PED Manual page ???."))
  )
  (if ss1
    (progn
      (setq form_ename (ssname ss1 0))
      (if
        (and
          (= (Gettag 11 form_ename) "DRAWING-NO")
          (= (Gettag 12 form_ename) "SHEET")
          (= (Gettag 13 form_ename) "REVISION")
        )
        (setq error_flag nil)
        (princ "\nError2: Proper drawing form (Ex: PEDFORM) is not present.\n        REV.LSP cannot continue.") ;        (princ "\nError2: Invalid titleblock.  See PED Manual page ???.")
  ) ) )
  (setvar "highlight" 1)
  (setq error_flag error_flag)
)

(princ "\r\\")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Enatt <number> <ename>) function
;
;Results:     Returns the entity name of the attribute at the level specified by
;          <number> within the block whose entity name is <ename>.

(defun Enatt (level ename / index)
  (setq index 1)
  (setq ename (entnext ename))
  (while (and (< index level) ename)
    (setq index (+ 1 index))
    (setq ename (entnext ename))
  )
  (setq ename ename)
)

(princ "\r|")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Getatt <number> <ename>) function
;
;References:  Enatt (located within this file)
;
;Results:     Returns the attribute value of the <number> attribute within the
;          <ename> block.  Output is in capital letters.

(defun Getatt (att_level blk_ename / ename)
  (setq ename (Enatt att_level blk_ename))
  (if ename
    (if (= (cdr (assoc 0 (entget ename))) "ATTRIB")
      (strcase (cdr (assoc 1 (entget ename))))
) ) )

(princ "\r/")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Gettag <number> <ename>) function
;
;References:  Enatt (located within this file)
;
;Results:     Returns the attribute tag of the <number> attribute within the
;          <ename> block.  Output is in capital letters.

(defun Gettag (tag_level blk_ename / ename)
  (Setq ename (Enatt tag_level blk_ename))
  (if ename
    (if (= (cdr (assoc 0 (entget ename))) "ATTRIB")
      (strcase (cdr (assoc 2 (entget ename))))
) ) )

(princ "\r-")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Change_att <number> <ename> <string>) function
;
;References:  Enatt (located within this file)
;
;Results:     Returns updated entity list.  Changes the attribute value of the
;          <number> attribute within the <ename> block to the value contained in
;          <string>.

(defun Change_att (att_level blk_ename new_att / ename elist)
  (setq ename (Enatt att_level blk_ename))
  (if ename
    (if (= (cdr (assoc 0 (entget ename))) "ATTRIB")
      (progn
        (setq elist (entget ename))
        (setq elist (subst (cons 1 new_att) (assoc 1 elist) elist))
        (entmod elist)
        (entupd blk_ename)
) ) ) )

(princ "\r-")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Query_string <number> <number> <string>) function
;
;Results:     Returns a text string input by the user.  The two <number>
;          parameters are the minimum and maximum number of characters
;          respectively.  The <string> is the prompt to the user.

(defun Query_string (lower_lim upper_lim prmpt / line)
  (setq line (strcase (getstring 1 prmpt)))
  (while (or (< (strlen line) lower_lim) (> (strlen line) upper_lim))
    (setq line (strcase (getstring 1 (strcat
      "Must be between " (itoa lower_lim) " and " (itoa upper_lim) " characters.\n" prmpt))))
  )
  (setq line line)
)

(princ "\r\\")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Query_int_nil <number> <number> <string>) function
;
;Results:     Retuns an integer value input by the user. Nil is a valid
;          responce.  The two <number> parameters are the minimum and maximum
;          input values respectively.  The <string> is the prompt to the user.

(defun Query_int_nil (lower_lim upper_lim prmpt / line)
  (setq line (getint prmpt))
  (while (and (or (< line lower_lim) (> line upper_lim)) (/= line nil))
    (setq line (getint (strcat
      "Must be a number between " (itoa lower_lim) " and " (itoa upper_lim) ".\n" prmpt)))
  )
  (setq line line)
)

(princ "\r|")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Get_date) function
;
;Results:     Returns the current date in the following format: "DAY MONTH YEAR"
;          The day and year are each numeric strings while the month is a two
;          letter abbreviation.

(defun Get_date (/ date month)
  (setq date (rtos (getvar "cdate") 2 0))
  (setq month (substr date 5 2))
  (cond
    ((= month "01") (setq month "JA"))
    ((= month "02") (setq month "FE"))
    ((= month "03") (setq month "MR"))
    ((= month "04") (setq month "AP"))
    ((= month "05") (setq month "MY"))
    ((= month "06") (setq month "JN"))
    ((= month "07") (setq month "JL"))
    ((= month "08") (setq month "AU"))
    ((= month "09") (setq month "SE"))
    ((= month "10") (setq month "OC"))
    ((= month "11") (setq month "NO"))
    ((= month "12") (setq month "DE"))
    (T (setq month "  "))
  )
  (setq date (strcat (substr date 7 2) month (substr date 3 2)))
)

(princ "\r/")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;($error$ <string>) function
;
;Results:     Returns the system error message <msg>.  UNDO BACK is performed
;          to the last marker set by the routine which errored out.

(defun $error$ (msg)
  (princ (strcat "\nError: " msg))
  (princ "\nUNDOing to point just before command was executed.")
  (setq *error* #error#)
  (command nil nil nil "undo" "b")
  (setvar "highlight" highlight)
  (princ)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(princ "\rC:REV")
(princ)