; ---------------------------------------------------------------------------
; STARTUP.ASM - startup code for HyperPAD extensions.
;
;		Last Modified: 12/07/90
;
; ---------------------------------------------------------------------------

DOS_PRINT_STRING	equ		09h
DOS_TERM_PROCESS	equ		4ch
INT_DOS			equ		21h

DOSSEG
.MODEL		LARGE				;child program must be large

.DATA

		public		__fac
__fac		db		8 dup ('F')	;floating accumulator

exit_msg	db		'This program requires HyperPAD.',13,10,'$'
		EXTRN		Pool:DWORD	;declared by child program
		EXTRN		ExtFlags:BYTE	;declared by child program, if wanted

XIFB		segment	byte	public	'DATA'
xistart		label	byte
XIFB		ends
XIF		segment	byte	public	'DATA'
XIF		ends
XIFE		segment	byte	public	'DATA'
xiend		label	byte
XIFE		ends

XCFB		segment	byte	public	'DATA'
xcstart		label	byte
XCFB		ends
XCF		segment	byte	public	'DATA'
XCF		ends
XCFE		segment	byte	public	'DATA'
xcend		label	byte
XCFE		ends

dgroup	group   _data,XIFB,XIF,XIFE,XCFB,XCF,XCFE

.CODE
		public		__acrtused
__acrtused	=		09876h  		;solve C startup reference
				       		; so regular startup code
				       		; doesn't get linked

		public		__fltused
__fltused	=		0987h

		public		FIARQQ
		public		FICRQQ
		public		FIDRQQ
		public		FIERQQ
		public		FISRQQ
		public		FIWRQQ
		public		FJARQQ
		public		FJCRQQ
		public		FJSRQQ

FIARQQ		equ		0fe32h
FICRQQ		equ		00e32h
FIDRQQ		equ		05c32h
FIERQQ		equ		01632h
FISRQQ		equ		00632h
FIWRQQ		equ		0a23dh
FJARQQ		equ		04000h
FJCRQQ		equ		0c000h
FJSRQQ		equ		08000h

		EXTRN		WhenLoaded:FAR		;declared in child
		EXTRN		WhenUnLoaded:FAR	;declared in child

; ---------------------------------------------------------------------------
; This table is the initial communication area between the host and the child.
; This table is pointed to by the word before the CS:IP in the exe header.
; When loaded, the host replaces info_area[0] with the actual address of the
; callback routine that performs DS and stack context switching.
; The host then reads the location of the child's function pool and the two
; required functions, WhenLoaded() and WhenUnLoaded().
;
; This area can be expanded by adding to the end.
; ---------------------------------------------------------------------------
info_area:	db		0ECh			;ID for an extension
rtn_array:	dd		0			;pointer to array of routines
		dd		Pool			;pointer to the function pool
		dd		__WhenLoaded		;pointer to WhenLoaded function
		dd		__WhenUnLoaded		;pointer to WhenUnLoaded function
		dw		DGROUP			;used to switch to DGROUP for the extension
		dd		ExtFlags		;pointer to flags BYTE

; ---------------------------------------------------------------------------
; startup - CS:IP in the exe header points here!  This will only receive
;           control if the user runs the extension from DOS!
;	    ...thus its purpose should be to display an error and quit - like
;	    MS windows programs.
; ---------------------------------------------------------------------------
		dd		info_area		;point to the info_area
startup		proc		far
		mov		ax,DGROUP		;segment of data
		mov		ds,ax
		mov		dx,offset exit_msg	;DS:DX point to string
		mov		ah,DOS_PRINT_STRING	;print error message
		int		INT_DOS

		mov		ax,(DOS_TERM_PROCESS SHL 8) + 08h
		int		INT_DOS			;exit to DOS
startup		endp

; ---------------------------------------------------------------------------
; __WhenLoaded: This function is called before the user's WhenLoaded function
; to perform any startup stuff that may be required. The GRAPHICS.LIB uses
; this method to set up video mode variables.
; ---------------------------------------------------------------------------
__WhenLoaded	proc		far
		push		ds
		push		si
		push		di

		mov		ax,@DATA
		mov		ds,ax
		mov		si,offset DGROUP:xistart
		mov		di,offset DGROUP:xiend
		call		initterm

		pop		di
		pop		si
		call		WhenLoaded
		pop		ds		;wait until AFTER WhenLoaded is called before setting stack back
		ret
__WhenLoaded	endp

__WhenUnLoaded	proc		far
		push		ds
		push		si
		push		di
		mov		ax,@DATA
		mov		ds,ax
		mov		si,offset DGROUP:xcstart
		mov		di,offset DGROUP:xcend
		call		initterm
		pop		di
		pop		si
		call		WhenUnLoaded
		pop		ds		;wait until here to set the stack back
		ret
__WhenUnLoaded	endp

initterm:
		cmp		si,di		; are we done?
		jae		initdone	;   yes - no more

		sub		di,4
		mov		ax,[di]
		or		ax,[di+2]
		jz		initterm	; skip null procedures
		call dword ptr	[di]
		jmp short	initterm	; keep looping

initdone:	ret

; ---------------------------------------------------------------------------
; exec - calls the host dispatcher. This must be called with the index of
; the desired routine in AX:
;
;	mov 	ax,1
;	jmp	exec
;
; ---------------------------------------------------------------------------
		public		exec
exec:		xor		ah,ah
		jmp dword ptr	rtn_array

END		startup





